// David Pilling August 2016
// Most of code from 
// http://forum.arduino.cc/index.php?topic=155589.msg1166639#msg1166639





#include <stdarg.h>

void pr(char *fmt, ... )
{
  char buf[128]; // resulting string limited to 128 chars
  va_list args;
  va_start (args, fmt );
  vsnprintf(buf, 128, fmt, args);
  va_end (args);
  Serial.println(buf);
}




#include "Arduino.h"

volatile unsigned t1captured = 0;
volatile unsigned t1capval = 0;
volatile unsigned t1ovfcnt = 0;
volatile unsigned long t1time;
volatile unsigned long t1last = 0;

#define BUFFER_SIZE 256

volatile unsigned long int buffer[BUFFER_SIZE];
volatile byte head = 0;
volatile byte tail = 0;


void setup() {

 Serial.begin(230400);  
 while (!Serial) { ; /* wait for serial port to connect. Needed for native USB */}

 TCCR1A = 0x0;    // put timer1 in normal mode

//ICNC1: Input Capture Noise Canceler
//ICES1: Input Capture Edge Select
//ICES1 bit is written to zero, a falling (negative) edge is used as trigger, 
//and when the ICES1 bit is written to one, a rising (positive) edge will trigger the capture.

 
 TCCR1B = 0x1;    // change prescaler to divide clock by 8

 // clear any pending capture or overflow interrupts
 TIFR1 = (1<<ICF1) | (1<<TOV1);
 // Enable input capture and overflow interrupts
 TIMSK1 |= (1<<ICIE1) | (1<<TOIE1);
 
 pinMode(8, INPUT);   //  Feed the signal in here 328p ProMini, Uno
 pinMode(4, INPUT);   //  Feed the signal in here 32u4 ProMicro Leonardo
}


static unsigned long int xcount=0;



static unsigned long int timex;

void loop() 
{
 byte newhead;

 while(head!=tail) 
 {
  newhead = (head + 1) % BUFFER_SIZE;
  Serial.println(buffer[newhead]);
  head = newhead;
 }
}



ISR(TIMER1_OVF_vect) { 
  t1ovfcnt++;              // keep track of overflows
}

ISR(TIMER1_CAPT_vect) {
 
 unsigned long t1temp;

 // combine overflow count with capture value to create 32 bit count
 //  calculate how long it has been since the last capture
 //   stick the result in the global variable t1time in 1uS precision
 t1capval = ICR1;
 t1temp = ((unsigned long)t1ovfcnt << 16) | t1capval;
 t1time = (t1temp - t1last);
 t1last = t1temp;
 
 tail = (tail + 1) % BUFFER_SIZE;
 buffer[tail] = t1time;
}


